//
//  ViewController.swift
//  lockStarsApp
//
//  Created by Richard Fasani on 1/27/16.
//  Copyright © 2016 Lock Stars. All rights reserved.
//

import UIKit
import CocoaMQTT

class ViewController: UIViewController {
    
    var mqtt: CocoaMQTT?
    var loadingFlag: Bool?

    override func viewDidLoad() {
        super.viewDidLoad()
        // Do any additional setup after loading the view, typically from a nib.
    }
    
    override func viewDidAppear(animated: Bool) {
        // make sure that default values are synchronized
        mqtt = getMQTT()
        if (mqtt != nil) {
            mqtt!.connect()
        }
        NSUserDefaults.standardUserDefaults().synchronize()
        loadingFlag = false
    }

    override func didReceiveMemoryWarning() {
        super.didReceiveMemoryWarning()
        // Dispose of any resources that can be recreated.
    }

    // called when unlock button is pressed
    @IBAction func unlockButton(sender: AnyObject) {
        //        let mqtt = CocoaMQTT(clientId: "lockStars", host: "10.176.58.5", port: 1883)
//        let mqtt = getMQTT()
//        if mqtt!.connect() {
        if (checkConnection()) {
            let dialog = UIAlertController(title: "Unlock Door", message: nil, preferredStyle: UIAlertControllerStyle.Alert)
            dialog.addAction(UIAlertAction(title: "Unlock", style: UIAlertActionStyle.Default, handler:{
                (UIAlertAction) -> Void in
                let unlockTime = NSUserDefaults.standardUserDefaults().valueForKey("unlockTime")
                self.mqtt!.publish("Open_Door", withString: unlockTime as! String, qos: .QOS0)
                self.dismissViewControllerAnimated(true, completion: nil)
            }))
            dialog.addAction(UIAlertAction(title: "Cancel", style: UIAlertActionStyle.Cancel, handler: nil))
            presentViewController(dialog, animated: true, completion: nil)
        }
        else {
            mqttFailedMessage()
        }
    }
    
//    func unlockDoor() {
//        // get unlock time from settings
//        let unlockTime = NSUserDefaults.standardUserDefaults().valueForKey("unlockTime")!
//        if (mqtt != nil) {
//            mqtt!.publish("Open_Door", withString: unlockTime as! String, qos: .QOS0)
//        }
//        else {
//            mqttFailedMessage()
//        }
//        mqtt!.publish("Open_Door", withString: "yes", qos: .QOS0)
//         eventually change to send time door should be open
//   }
    @IBAction func connectMQTT(sender: AnyObject) {
//        mqtt = CocoaMQTT(clientId: "lockStars", host: "10.176.58.5", port: 1883)
//        let mqtt = getMQTT()
//        if mqtt!.connect() {
        if (mqtt != nil) {
            mqtt!.connect()
            if (mqtt!.connected) {
                let clientID = NSUserDefaults.standardUserDefaults().valueForKey("clientID") as! String
                let host = NSUserDefaults.standardUserDefaults().valueForKey("mqttServer") as! String
                let portString = NSUserDefaults.standardUserDefaults().valueForKey("mqttPort") as! String
                let dialog = UIAlertController(title: "Connection Successful", message: "client: \(clientID) \r\nhost: \(host) \r\nport: \(portString)", preferredStyle: UIAlertControllerStyle.Alert)
                    dialog.addAction(UIAlertAction(title: "Close", style: UIAlertActionStyle.Default,handler: nil))
                    presentViewController(dialog, animated: true, completion: nil)
            }
            
//            if (mqtt!.connected) {
//                let dialog = UIAlertController(title: "Success", message: "MQTT Connection Successful", preferredStyle: UIAlertControllerStyle.Alert)
//                dialog.addAction(UIAlertAction(title: "Close", style: UIAlertActionStyle.Default, handler: nil))
//                presentViewController(dialog, animated: true, completion: nil)
//                
//            }
//            else {
//                mqttFailedMessage()
//            }

        }
        else {
            mqttFailedMessage()
        }
    }
    
//    @IBAction func mqttTestMessage(sender: AnyObject) {
////        let mqtt = getMQTT()
////        if mqtt!.connect() {
//        if (mqtt != nil) {
//            let dialog = UIAlertController(title: "Test Message", message: "What string do you want to publish?", preferredStyle: UIAlertControllerStyle.Alert)
//            dialog.addTextFieldWithConfigurationHandler({ (textField) -> Void in textField.text = nil})
//            dialog.addAction(UIAlertAction(title: "Publish", style: UIAlertActionStyle.Default, handler: {  (UIAlertAction) -> Void in
//                let textField = dialog.textFields![0] as UITextField
//                self.mqtt!.publish("test", withString: textField.text!, qos: .QOS0)
//                self.dismissViewControllerAnimated(true, completion: nil)
//            }))
//            dialog.addAction(UIAlertAction(title: "Cancel", style: UIAlertActionStyle.Cancel, handler: nil))
//            presentViewController(dialog, animated: true, completion: nil)
//        }
//        else {
//            mqttFailedMessage()
//        }
////        mqtt!.publish("test", withString: "hello", qos: .QOS0)
//    }
    
    @IBAction func settingsButton(sender: AnyObject) {
        let settingsUrl = NSURL(string: UIApplicationOpenSettingsURLString)
        if let url = settingsUrl {
            UIApplication.sharedApplication().openURL(url)
        }
    }

//    var mqtt: CocoaMQTT?
    
    @IBAction func takePicture(sender: AnyObject) {
//        let mqtt = getMQTT()
//        mqtt!.publish("test", withString: "test", qos: .QOS0)
        if (checkConnection()) {
            mqtt!.publish("Take_Pic", withString: "yes", qos: .QOS0)
            loadingFlag = true
            performSegueWithIdentifier("ViewPictureSegue", sender: self)
        }
        else {
            mqttFailedMessage()
        }
    }

    
    func mqttFailedMessage() {
        let dialog = UIAlertController(title: "Failure", message: "MQTT Connection Failed", preferredStyle: UIAlertControllerStyle.Alert)
        dialog.addAction(UIAlertAction(title: "Close", style: UIAlertActionStyle.Default, handler: nil))
        presentViewController(dialog, animated: true, completion: nil)
    }
    
    func getMQTT() -> CocoaMQTT {
        if (checkMQTTSettings()) {
            let clientID = NSUserDefaults.standardUserDefaults().valueForKey("clientID") as! String
            let host = NSUserDefaults.standardUserDefaults().valueForKey("mqttServer") as! String
            let portString = NSUserDefaults.standardUserDefaults().valueForKey("mqttPort") as! String
            let port = UInt16(portString)!
        
            let mqtt = CocoaMQTT(clientId: clientID, host: host, port: port)
            mqtt.delegate = self
            return mqtt
        }
        else {
            settingsMissingDialog()
            let mqtt = CocoaMQTT(clientId: "", host: "", port: 0)
            return mqtt
        }
    }
    
    func checkMQTTSettings() -> Bool {
        if (NSUserDefaults.standardUserDefaults().valueForKey("clientID") == nil) {
            return false
        }
        else if (NSUserDefaults.standardUserDefaults().valueForKey("mqttServer") == nil) {
            return false
        }
        else if (NSUserDefaults.standardUserDefaults().valueForKey("mqttPort") == nil) {
            return false
        }
        else {
            return true
        }
    }
    
    func settingsMissingDialog() {
        let dialog = UIAlertController(title: "Error: MQTT Server Info", message: "MQTT server information is missing in user settings. Please update your settings.", preferredStyle: UIAlertControllerStyle.Alert)
        dialog.addAction(UIAlertAction(title: "Go to Settings", style: UIAlertActionStyle.Default, handler:{
            (UIAlertAction) -> Void in
            let settingsUrl = NSURL(string: UIApplicationOpenSettingsURLString)
            if let url = settingsUrl {
                UIApplication.sharedApplication().openURL(url)
            }
        }))
        dialog.addAction(UIAlertAction(title: "Close", style: UIAlertActionStyle.Cancel, handler: nil))
        presentViewController(dialog, animated: true, completion: nil)
    }
    
    func checkConnection() -> Bool {
        if (mqtt != nil) {
            if (mqtt!.connected) {
                return true
            }
            else {
                return false
            }
        }
        else {
            return false
        }
    }
    
    override func prepareForSegue(segue: UIStoryboardSegue, sender: AnyObject?) {
        if let identifier = segue.identifier {
            switch identifier {
            case "ViewPictureSegue":
                let vc = segue.destinationViewController as! ViewPictureViewController
                vc.mqtt = mqtt
                vc.loadingFlag = loadingFlag!
                mqtt?.disconnect()
            default:
                break
            }
        }
    }
}

//     being used to connect to MQTT Server now
//    @IBAction func dorbellCameraButton(sender: AnyObject) {
////        mqtt = CocoaMQTT(clientId: "lockStars", host: "10.176.58.5", port: 1883)
////        mqtt!.host = "10.176.58.5"
////        mqtt!.port = 1883
////        mqtt!.clientId = "lockStars"
////        print(mqtt!.connect())
//        
////        var message: CocoaMQTTMessage
////        message.topic = "test"
////        message.string = "hello!"
////        mqtt!.publish(message)
//    }


extension ViewController: CocoaMQTTDelegate {
    
    func mqtt(mqtt: CocoaMQTT, didConnect host: String, port: Int) {
        print("didConnect \(host):\(port)")
        mqtt.connected = true
//        let dialog = UIAlertController(title: "Connection Successful", message: "host: \(host) port: \(port)", preferredStyle: UIAlertControllerStyle.Alert)
//        dialog.addAction(UIAlertAction(title: "Close", style: UIAlertActionStyle.Default, handler: nil))
//        presentViewController(dialog, animated: true, completion: nil)
    }
    
    func mqtt(mqtt: CocoaMQTT, didConnectAck ack: CocoaMQTTConnAck) {
        //print("didConnectAck \(ack.rawValue)")
        if ack == .ACCEPT {
            mqtt.ping()
            
//            let chatViewController = storyboard?.instantiateViewControllerWithIdentifier("ChatViewController") as? ChatViewController
//            chatViewController?.mqtt = mqtt
//            navigationController!.pushViewController(chatViewController!, animated: true)
        }
        
    }
    
    func mqtt(mqtt: CocoaMQTT, didPublishMessage message: CocoaMQTTMessage, id: UInt16) {
        print("didPublishMessage with message: \(message.string)")
    }
    
    func mqtt(mqtt: CocoaMQTT, didPublishAck id: UInt16) {
        print("didPublishAck with id: \(id)")
    }
    
    func mqtt(mqtt: CocoaMQTT, didReceiveMessage message: CocoaMQTTMessage, id: UInt16 ) {
        print("didReceivedMessage: \(message.string) with id \(id)")
//        NSNotificationCenter.defaultCenter().postNotificationName("MQTTMessageNotification" + animal!, object: self, userInfo: ["message": message.string!, "topic": message.topic])
    }
    
    func mqtt(mqtt: CocoaMQTT, didSubscribeTopic topic: String) {
        print("didSubscribeTopic to \(topic)")
    }
    
    func mqtt(mqtt: CocoaMQTT, didUnsubscribeTopic topic: String) {
        print("didUnsubscribeTopic to \(topic)")
    }
    
    func mqttDidPing(mqtt: CocoaMQTT) {
        print("didPing")
    }
    
    func mqttDidReceivePong(mqtt: CocoaMQTT) {
        _console("didReceivePong")
        mqtt.connected = true
    }
    
    func mqttDidDisconnect(mqtt: CocoaMQTT, withError err: NSError?) {
        _console("mqttDidDisconnect")
//        let dialog = UIAlertController(title: "MQTT Disconnected", message: "Disconnected from MQTT Server", preferredStyle: UIAlertControllerStyle.Alert)
//        dialog.addAction(UIAlertAction(title: "Close", style: UIAlertActionStyle.Default, handler: nil))
//        presentViewController(dialog, animated: true, completion: nil)
        mqtt.connected = false
    }
    
    func _console(info: String) {
        print("Delegate: \(info)")
    }
    
}

