//
//  ViewPictureViewController.swift
//  lockStarsApp
//
//  Created by Richard Fasani on 4/19/16.
//  Copyright © 2016 Lock Stars. All rights reserved.
//

import UIKit
import CocoaMQTT

class ViewPictureViewController: UIViewController {
    
    @IBOutlet weak var unlockDoorButton: UIBarButtonItem!
    @IBOutlet weak var takePicButton: UIBarButtonItem!
    @IBOutlet weak var web: UIWebView!
    @IBOutlet weak var activityIndicator: UIActivityIndicatorView!
    var mqtt: CocoaMQTT?
    lazy var timer = NSTimer()
    var seconds = 0
    var loadingFlag: Bool?
    
    override func viewDidLoad() {
        super.viewDidLoad()
        if (loadingFlag!) {
            loadingFunc()
        }
        else {
            loadWeb()
        }
//        mqtt = getMQTT()
        if (mqtt != nil) {
            mqtt!.connect()
        }
        // Do any additional setup after loading the view.
    }

    override func didReceiveMemoryWarning() {
        super.didReceiveMemoryWarning()
        // Dispose of any resources that can be recreated.
    }
    
    @IBAction func takePicture(sender: AnyObject) {
//        let mqtt = CocoaMQTT(clientId: "lockStars", host: "10.176.58.5", port: 1883)
//        let mqtt = getMQTT()
//        if mqtt.connect() {
        if (checkConnection()) {
            let dialog = UIAlertController(title: "Take Picture", message: nil, preferredStyle: UIAlertControllerStyle.Alert)
            dialog.addAction(UIAlertAction(title: "Take Picture", style: UIAlertActionStyle.Default, handler:{
                (UIAlertAction) -> Void in
                self.mqtt!.publish("Take_Pic", withString: "yes", qos: .QOS0)
                self.dismissViewControllerAnimated(true, completion: nil)
                self.loadingFunc()
//                self.timer = NSTimer.scheduledTimerWithTimeInterval(1, target: self, selector: "timerShot:", userInfo: nil, repeats: true)
//                self.web.hidden = true
//                self.activityIndicator.startAnimating()
            }))
            dialog.addAction(UIAlertAction(title: "Cancel", style: UIAlertActionStyle.Cancel, handler: nil))
            presentViewController(dialog, animated: true, completion: nil)
        }
        else {
            mqttFailedMessage()
        }
    }
    
    @IBAction func unlockDoor(sender: AnyObject) {
//        let mqtt = CocoaMQTT(clientId: "lockStars", host: "10.176.58.5", port: 1883)
//        let mqtt = getMQTT()
//        if mqtt.connect() {
        if (checkConnection()) {
            let dialog = UIAlertController(title: "Unlock Door", message: nil, preferredStyle: UIAlertControllerStyle.Alert)
            dialog.addAction(UIAlertAction(title: "Unlock", style: UIAlertActionStyle.Default, handler:{
                (UIAlertAction) -> Void in
                let unlockTime = NSUserDefaults.standardUserDefaults().valueForKey("unlockTime")
                self.mqtt!.publish("Open_Door", withString: unlockTime as! String, qos: .QOS0)
                self.dismissViewControllerAnimated(true, completion: nil)
            }))
            dialog.addAction(UIAlertAction(title: "Cancel", style: UIAlertActionStyle.Cancel, handler: nil))
            presentViewController(dialog, animated: true, completion: nil)
        }
        else {
            mqttFailedMessage()
        }
    }
    
    func loadingFunc() {
        takePicButton.enabled = false
        unlockDoorButton.enabled = false
        timer = NSTimer.scheduledTimerWithTimeInterval(1, target: self, selector: "timerShot:", userInfo: nil, repeats: true)
        web.hidden = true
        activityIndicator.startAnimating()
    }
    
    func loadWeb() {
        let urlString = "http://sd.lockstars.com.s3-website-us-west-2.amazonaws.com/"
        let requestURL = NSURL(string: urlString)
        let request = NSURLRequest(URL: requestURL!)
        web.loadRequest(request)
    }
    
    func mqttFailedMessage() {
        let dialog = UIAlertController(title: "Failure", message: "MQTT Connection Failed", preferredStyle: UIAlertControllerStyle.Alert)
        dialog.addAction(UIAlertAction(title: "Close", style: UIAlertActionStyle.Default, handler: nil))
        presentViewController(dialog, animated: true, completion: nil)
    }
    
    func getMQTT() -> CocoaMQTT {
        if (checkMQTTSettings()) {
            let clientID = NSUserDefaults.standardUserDefaults().valueForKey("clientID") as! String
            let host = NSUserDefaults.standardUserDefaults().valueForKey("mqttServer") as! String
            let portString = NSUserDefaults.standardUserDefaults().valueForKey("mqttPort") as! String
            let port = UInt16(portString)!
            
            let mqtt = CocoaMQTT(clientId: clientID, host: host, port: port)
            mqtt.delegate = self
            return mqtt
        }
        else {
            settingsMissingDialog()
            let mqtt = CocoaMQTT(clientId: "", host: "", port: 0)
            return mqtt
        }
    }
    
    func checkMQTTSettings() -> Bool {
        if (NSUserDefaults.standardUserDefaults().valueForKey("clientID") == nil) {
            return false
        }
        else if (NSUserDefaults.standardUserDefaults().valueForKey("mqttServer") == nil) {
            return false
        }
        else if (NSUserDefaults.standardUserDefaults().valueForKey("mqttPort") == nil) {
            return false
        }
        else {
            return true
        }
    }
    
    func settingsMissingDialog() {
        let dialog = UIAlertController(title: "Error: MQTT Server Info", message: "MQTT server information is missing in user settings. Please update your settings.", preferredStyle: UIAlertControllerStyle.Alert)
        dialog.addAction(UIAlertAction(title: "Go to Settings", style: UIAlertActionStyle.Default, handler:{
            (UIAlertAction) -> Void in
            let settingsUrl = NSURL(string: UIApplicationOpenSettingsURLString)
            if let url = settingsUrl {
                UIApplication.sharedApplication().openURL(url)
            }
        }))
        dialog.addAction(UIAlertAction(title: "Close", style: UIAlertActionStyle.Cancel, handler: nil))
        presentViewController(dialog, animated: true, completion: nil)
    }
    
    func timerShot(timer: NSTimer) {
        seconds++
        if (seconds >= 8) {
            activityIndicator.stopAnimating()
            loadWeb()
            web.hidden = false
            timer.invalidate()
            seconds = 0
            takePicButton.enabled = true
            unlockDoorButton.enabled = true
        }
    }
    
    func checkConnection() -> Bool {
        if (mqtt != nil) {
            if (mqtt!.connected) {
                return true
            }
            else {
                return false
            }
        }
        else {
            return false
        }
    }
}

extension ViewPictureViewController: CocoaMQTTDelegate {
    
    func mqtt(mqtt: CocoaMQTT, didConnect host: String, port: Int) {
        print("didConnect \(host):\(port)")
    }
    
    func mqtt(mqtt: CocoaMQTT, didConnectAck ack: CocoaMQTTConnAck) {
        //print("didConnectAck \(ack.rawValue)")
        if ack == .ACCEPT {
            mqtt.ping()
            
            //            let chatViewController = storyboard?.instantiateViewControllerWithIdentifier("ChatViewController") as? ChatViewController
            //            chatViewController?.mqtt = mqtt
            //            navigationController!.pushViewController(chatViewController!, animated: true)
        }
        
    }
    
    func mqtt(mqtt: CocoaMQTT, didPublishMessage message: CocoaMQTTMessage, id: UInt16) {
        print("didPublishMessage with message: \(message.string)")
    }
    
    func mqtt(mqtt: CocoaMQTT, didPublishAck id: UInt16) {
        print("didPublishAck with id: \(id)")
    }
    
    func mqtt(mqtt: CocoaMQTT, didReceiveMessage message: CocoaMQTTMessage, id: UInt16 ) {
        print("didReceivedMessage: \(message.string) with id \(id)")
        //        NSNotificationCenter.defaultCenter().postNotificationName("MQTTMessageNotification" + animal!, object: self, userInfo: ["message": message.string!, "topic": message.topic])
    }
    
    func mqtt(mqtt: CocoaMQTT, didSubscribeTopic topic: String) {
        print("didSubscribeTopic to \(topic)")
    }
    
    func mqtt(mqtt: CocoaMQTT, didUnsubscribeTopic topic: String) {
        print("didUnsubscribeTopic to \(topic)")
    }
    
    func mqttDidPing(mqtt: CocoaMQTT) {
        print("didPing")
    }
    
    func mqttDidReceivePong(mqtt: CocoaMQTT) {
        _console("didReceivePong")
        mqtt.connected = true
    }
    
    func mqttDidDisconnect(mqtt: CocoaMQTT, withError err: NSError?) {
        _console("mqttDidDisconnect")

    }
    
    func _console(info: String) {
        print("Delegate: \(info)")
    }
    
}
